@@ -5,8 +5,15 @@ Unit tests for the helper funcs module.
 import unittest
 import numpy as np
 import numpy.testing as npt
+import datetime
 
-from nessie.helper_funcs import create_density_function, calculate_s_total, remap_ids
+
+from nessie.helper_funcs import (
+    create_density_function,
+    calculate_s_total,
+    remap_ids,
+    gen_random_redshifts,
+)
 from nessie.cosmology import FlatCosmology
 
 
@@ -90,5 +97,26 @@ class TestRemapId(unittest.TestCase):
             self.assertEqual(r, a)
 
 
+class TestGenRandoms(unittest.TestCase):
+    """
+    Testing that the gen_random_redshifts function.
+    """
+
+    def test(self):
+        """testing basic run since this is a simple wrapper."""
+        cosmo = FlatCosmology(1.0, 0.3)
+        redshifts = np.random.normal(0.6, 0.3, 500)
+        redshifts = redshifts[redshifts > 0]
+        mags = np.random.random(len(redshifts)) * 4 + 15
+        z_lim = np.max(redshifts) + 0.1
+        maglim = 19.0
+        print("Doing this thing")
+        now = datetime.datetime.now()
+        zs = gen_random_redshifts(redshifts, mags, z_lim, maglim, cosmo, iterations=2, n_clone=100)
+        later = datetime.datetime.now()
+        print(f"time taken: {later - now}")
+        self.assertEqual(round(len(zs) / (100*500)), 1)
+
+
 if __name__ == "__main__":
     unittest.main()
